from evoagent.llms.llms import GeminiLLM
from evoagent.embeddings import GeminiEmbeddings
from evoagent.vector_stores import FaissVectorStore
from evoagent.tools.file_reader import DocumentReader
from typing import List
import textwrap
from dotenv import load_dotenv
import os

class RAGApplication:
    def __init__(self, 
                 model_name: str = "gemini-1.5-flash",
                 chunk_size: int = 1000, 
                 chunk_overlap: int = 200):
        """
        Initialize RAG application with EvoAgent components.
        
        Args:
            model_name (str): Name of the Gemini model to use
            chunk_size (int): Size of text chunks for processing
            chunk_overlap (int): Overlap between chunks
        """
        # Load environment variables
        load_dotenv()
        
        # Validate API key
        if not os.getenv("GOOGLE_API_KEY"):
            raise ValueError("Please set GOOGLE_API_KEY in your .env file")
        
        # Initialize components
        self.llm = GeminiLLM(model_name)
        self.embeddings = GeminiEmbeddings()
        self.reader = DocumentReader()
        
        # Initialize vector store with proper dimension
        sample_embedding = self.embeddings.embed("Sample text").values
        self.vector_store = FaissVectorStore(embedding_dim=len(sample_embedding))
        
        self.chunk_size = chunk_size
        self.chunk_overlap = chunk_overlap
        
        # Keep track of indexed documents
        self.indexed_docs = set()
    
    def chunk_text(self, text: str) -> List[str]:
        """Split text into overlapping chunks."""
        chunks = []
        start = 0
        
        while start < len(text):
            end = start + self.chunk_size
            chunk = text[start:end]
            
            if start > 0:
                chunk = text[start - self.chunk_overlap:end]
            
            chunk = chunk.strip()
            if chunk:
                chunks.append(chunk)
            
            start += self.chunk_size - self.chunk_overlap
            
        return chunks
    
    def index_document(self, file_path: str) -> None:
        """
        Read and index a document.
        
        Args:
            file_path (str): Path to the document
        """
        if not os.path.exists(file_path):
            raise FileNotFoundError(f"Document not found: {file_path}")
            
        if file_path in self.indexed_docs:
            print(f"Document {file_path} already indexed")
            return
            
        try:
            # Read document
            text = self.reader.read(file_path)
            
            # Split into chunks
            chunks = self.chunk_text(text)
            
            # Generate embeddings for chunks
            embeddings_list = [self.embeddings.embed(chunk).values for chunk in chunks]
            
            # Store in vector store
            self.vector_store.add_batch(chunks, embeddings_list)
            
            # Mark document as indexed
            self.indexed_docs.add(file_path)
            
            print(f"Successfully indexed {len(chunks)} chunks from {file_path}")
            
        except Exception as e:
            print(f"Error indexing document {file_path}: {str(e)}")
    
    def query(self, 
              question: str, 
              num_chunks: int = 3,
              temperature: float = 0.7) -> str:
        """
        Query the RAG system.
        
        Args:
            question (str): User question
            num_chunks (int): Number of relevant chunks to retrieve
            temperature (float): Temperature for LLM response generation
            
        Returns:
            str: Generated answer
        """
        try:
            # Generate embedding for question
            query_embedding = self.embeddings.embed(question).values
            
            # Retrieve relevant chunks
            relevant_chunks = self.vector_store.query(query_embedding, k=num_chunks)
            
            # Generate prompt with context
            prompt = self._generate_prompt(question, relevant_chunks)
            
            # Get response from LLM
            response = self.llm.generate(prompt, temperature=temperature)
            
            return response
            
        except Exception as e:
            return f"Error generating response: {str(e)}"
    
    def _generate_prompt(self, query: str, context: List[str]) -> str:
        """Generate prompt with retrieved context."""
        return f"""Use the following passages to answer the question. If you cannot answer based on the passages, say "I cannot answer this question based on the provided information."

Relevant passages:
{'-' * 80}
{' '.join(context)}
{'-' * 80}

Question: {query}

Answer:"""

def main():
    # Initialize RAG application
    rag = RAGApplication()
    
    # Index documents
    documents = [
        "path/to/your/document1.pdf",
        "path/to/your/document2.txt"
    ]
    
    print("Indexing documents...")
    for doc in documents:
        rag.index_document(doc)
    
    # Interactive query loop
    print("\nEnter your questions (type 'exit' to quit):")
    while True:
        question = input("\nQuestion: ").strip()
        if question.lower() == 'exit':
            break
            
        response = rag.query(question)
        print("\nAnswer:", textwrap.fill(response, width=80))

if __name__ == "__main__":
    main()